--[[ Copyright (c) 2013 Luis Pais

This software is provided 'as-is', without any express or implied
warranty. In no event will the authors be held liable for any damages
arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

   1. The origin of this software must not be misrepresented; you must not
   claim that you wrote the original software. If you use this software
   in a product, an acknowledgment in the product documentation would be
   appreciated but is not required.

   2. Altered source versions must be plainly marked as such, and must not be
   misrepresented as being the original software.

   3. This notice may not be removed or altered from any source
   distribution.
]]

function resizePoint(obj1,px,py)
	 if not isEmpty(obj1) then
		 if px~=nil then
			 if obj1.x < px then --checks if obj1.x is to the left of px
				 obj1.width=px-obj1.x
			 end
			 if obj1.x > px then --checks if obj1.x is to the right of px
				 local obj1x=obj1.x
				 obj1.x=px
				 obj1.width=obj1x-obj1.x
			 end
		 end
		 if py~=nil then
			 if obj1.y < py then --checks if py is under obj1.y
				 obj1.height=py-obj1.y
			 end
			 if obj1.y > py then --checks if obj1.y is under py
				 local obj1y=obj1.y
				 obj1.y=py
				 obj1.height=obj1y-obj1.y
			 end
		 end
	 end
end

function reshapePoint(obj1,originx,originy,currx,curry)
	 if not isEmpty(obj1) then
		 if originx~=nil then
			 if originx < currx then --checks if obj1.x is to the left of px
				 obj1.width=currx-originx
			 end
			 if originx > currx then --checks if obj1.x is to the right of px
				 obj1.x=currx
				 obj1.width=originx-currx
			 end
		 end
		 if originy~=nil then
			 if originy < curry then --checks if py is under obj1.y
				 obj1.height=curry-originy
			 end
			 if obj1.y > py then --checks if obj1.y is under py
				 obj1.y=curry
				 obj1.height=originy-curry
			 end
		 end
	 end
end

function returnRect(obj1)
	 return obj1.x, obj1.y, obj1.width, obj1.height
end

function newRect(x,y,w,h)
	 local newrectangle={}
	 newrectangle.x=x
	 newrectangle.y=y
	 newrectangle.width=w
	 newrectangle.height=h
	 return newrectangle
end

function moveRect(obj1,mx,my)
	 mx=mx or obj1.x
	 my=my or obj1.y

	 obj1.x=mx
	 obj1.y=my
end

function translateRect(obj1,tx,ty,speed,dt,debugger)
	 if obj1.x==tx and obj1.y==ty then return true end
	 local tx=tx or obj1.x
	 local ty=ty or obj1.y
	 local dt=dt or 1
	 local did=false
	 --print(obj1.x,obj1.y,tx,ty)
	 if obj1.x < tx then --detects if obj1 is to the left of tx
		 if obj1.x+speed*dt > tx then
			 obj1.x=tx
			 did=true
		 else
			 obj1.x = obj1.x + speed*dt
			 did=true
		 end
	 elseif obj1.x > tx then --detect if it is to the right
		 if obj1.x-speed*dt < tx then
			 obj1.x=tx
			 did=true
		 else
			 obj1.x = obj1.x - speed*dt
			 did=true
		 end
	 end
	 if obj1.y < ty then --detects if ty is under obj1
		 if obj1.y +speed*dt > ty then
			 obj1.y = ty
			 did=true
		 else
			 --print('translated '..debugger..' down')
			 obj1.y = obj1.y + speed*dt
			 did=true
		 end
	 elseif obj1.y > ty then --detects if obj1.y is under ty
		 if obj1.y +speed*dt < ty then
			 obj1.y = ty
			 did=true
		 else
			 --print('translated '..debugger..' up')
			 obj1.y = obj1.y - speed*dt
			 did=true
		 end
	 end
	 if did then return false else return true end
end	  

function resizeRect(obj1,gw,gh)
	 gw=gw or obj1.width
	 gh=gh or obj1.height
	 obj1.width = gw
	 obj1.height = gh
end
 
function reshapeRect(obj1,rx,ry,rw,rh)
	 rx=rx or obj1.x
	 ry=ry or obj1.y
	 rw=rw or obj1.width
	 rh=rh or obj1.height
	 obj1.x=rx
	 obj1.y=ry
	 obj1.width=rw
	 obj1.height=rh
end

function getArea(obj1)
	 return obj1.width*obj1.height
end

function getInter(a, b)
    local left = math.max(a.x, b.x)
    local right = math.min(a.x + a.width, b.x + b.width)
    local top = math.max(a.y, b.y)
    local bottom = math.min(a.y + a.height, b.y + b.height)
    if left < right and top < bottom then
        return newRect(left, top, right - left, bottom - top)
	else
		 return newRect(0,0,0,0)
    end
end

function isInter(obj1,obj2,tolerance)
	 tolerance=tolerance or 0
	 if isEmpty(obj1) or isEmpty(obj2) then
		 return false
	 end
	 if obj1.x+obj1.width-tolerance < obj2.x or obj1.x+tolerance > obj2.x+obj2.width or obj1.y+obj1.height-tolerance < obj2.y or obj1.y-tolerance > obj2.y+obj2.height then
		 return false
	 else
		 return true
	 end
end

function isIn(obj1,px,py)
	 if px > obj1.x and px < obj1.x+obj1.width and py > obj1.y and py < obj1.y+obj1.height then
		 return true
	 else
		 return false
	 end
end

function getUnion(obj1,obj2)
	 local unionRect = {}
	 unionRect.x1=math.min(obj1.x,obj2.x)
	 unionRect.x2=math.max(obj1.x+obj1.width,obj2.x+obj2.width)
	 unionRect.y1=math.min(obj1.y,obj2.y)
	 unionRect.y2=math.max(obj1.y+obj1.height, obj2.y+obj2.height)
	 if not isEmpty(obj1) and not isEmpty(obj2) then
		 return newRect(unionRect.x1,unionRect.y1,unionRect.x2-unionRect.x1,unionRect.y2-unionRect.y1)
	 else
		 return newRect(0,0,0,0)
	 end
end

function equalsRect(obj1,obj2)
	 if obj1.x==obj2.x and obj1.y==obj2.y and obj1.width==obj2.width and obj1.height==obj2.height then
		 return true
	 else
		 return false
	 end
end

function emptyRect(obj1)
	 obj1.x=0
	 obj1.y=0
	 obj1.width=0
	 obj1.height=0
end

function isEmpty(obj1)
	 if (obj1==nil) or (obj1.x==nil and obj1.y==nil and obj1.width==nil and obj1.height==nil) or (obj1.width <=0 and obj1.height <= 0) then
		 return true
	 else
		 return false
	 end
end

function tostringRect(obj1,rectname)
	 if rectname == nil then
		 return 'x='..obj1.x..',y='..obj1.y..',width='..obj1.width..',height='..obj1.height
	 else
		 return rectname..'.x='..obj1.x..', '..rectname..'.y='..obj1.y..', '..rectname..'.width='..obj1.width..', '..rectname..'.height='..obj1.height
	 end
end